@@ -7,8 +7,11 @@ use Carp;
 use warnings;
 no warnings qw(uninitialized);
 our @ISA;
-use vars qw($n %by_index $fade_down_fraction
-$fade_time1_fraction $fade_time2_fraction $fader_op);
+use vars qw($n %by_index);
+use Audio::Nama::Globals qw(:singletons %tn @fade_data); 
+use Audio::Nama::Log qw(logsub logpkg);
+use Audio::Nama::Effect  qw(remove_effect add_effect update_effect);
+# we don't import 'type' as it would clobber our $fade->type attribute
 use Audio::Nama::Object qw( 
 				 n
 				 type
@@ -21,17 +24,9 @@ use Audio::Nama::Object qw(
 				 );
 initialize();
 
-# example
-#
-# if fade time is 10 for a fade out
-# and fade start time is 0:
-#
-# from 0 to 9, fade from 0 (100%) to -64db
-# from 9 to 10, fade from -64db to -256db
-
 sub initialize { 
 	%by_index = (); 
-	@Audio::Nama::fade_data = (); # for save/restore
+	@fade_data = (); # for save/restore
 }
 sub next_n {
 	my $n = 1;
@@ -46,22 +41,20 @@ sub new {
 	my $object = bless 
 	{ 
 #		class => $class,  # not needed yet
-		n => next_n(), 
+		n => next_n(),    
 		relation => 'fade_from_mark',
 		@_	
 	}, $class;
 
 	$by_index{$object->n} = $object;
 
-	#print "object class: $class, object type: ", ref $object, $/;
+	logpkg(__FILE__,__LINE__,'debug',"object class: $class, object type: ", ref $object);
 
-	my $id = add_fader($object->track);	# only when necessary
+	my $id = add_fader($object->track);
 	
-	my $track = $Audio::Nama::tn{$object->track};
+	my $track = $tn{$object->track};
 
-	# add linear envelope controller -klg if needed
-	
-	refresh_fade_controller($track);
+	Audio::Nama::request_setup(); # runs apply_fades and reconfigures engine
 	$object
 	
 }
@@ -70,79 +63,68 @@ sub new {
 
 sub refresh_fade_controller {
 	my $track = shift;
-	my %mute_level = (ea => 0, 	eadb => -256); 
-	my $operator  = $Audio::Nama::cops{$track->fader}->{type};
-	my $off_level = $mute_level{$operator};
-	my $on_level  = $Audio::Nama::unity_level{$operator};
-
-	# remove controller if present
-	if( $track->fader and my ($old) = @{$Audio::Nama::cops{$track->fader}{owns}})
-		{ Audio::Nama::remove_effect($old) }
-
-	return unless
-		my @pairs = fader_envelope_pairs($track); 
-
-	# add fader if it is missing
-
+	my @pairs = fader_envelope_pairs($track);
 	add_fader($track->name);	
+	my $operator  = Audio::Nama::fxn($track->fader)->type;
+	my $off_level = $config->{mute_level}->{$operator};
+	my $on_level  = $config->{unity_level}->{$operator};
+	my @controllers = @{Audio::Nama::fxn($track->fader)->owns};
+	logpkg(__FILE__,__LINE__,'debug',$track->name, ": existing controllers: @controllers");
+	for my $controller (@controllers)
+	{
+		logpkg(__FILE__,__LINE__,'debug',"removing fade controller $controller");
+		remove_effect($controller);
+	}
 
 	# add controller
-	Audio::Nama::Text::t_add_ctrl($track->fader,  # parent
-					 'klg',	  		 # Ecasound controller
-					 [1,				 # Ecasound parameter 1
-					 $off_level,
-					 $on_level,
-					 @pairs,
-					 ]
-	);
+	my $reuseid = pop @controllers; # we expect only one
+	logpkg(__FILE__,__LINE__,'debug',"applying fade controller");
+	add_effect({
+		track		=> $track,
+		id			=> $reuseid,
+		parent	 	=> $track->fader,
+		type		=> 'klg',	  		 # Ecasound controller
+		params => [	1,				 # modify first parameter of fader op 
+					 		$off_level,
+					 		$on_level,
+					 		@pairs,
+					 	]
+	});
 
 	# set fader to correct initial value
 	# 	first fade is type 'in'  : 0
 	# 	first fade is type 'out' : 100%
 	
-	 
-	Audio::Nama::effect_update_copp_set($track->fader,0, initial_level($track->name))
+	update_effect($track->fader,0, initial_level($track->name) * 100)
 }
 
 
 sub all_fades {
 	my $track_name = shift;
-	grep{ $_->track eq $track_name } values %by_index
+	sort { 
+		$Audio::Nama::Mark::by_name{$a->mark1}->{time} <=> $Audio::Nama::Mark::by_name{$b->mark1}->{time}
+	} grep { $_->track eq $track_name } values %by_index
 }
 sub fades {
 
 	# get fades within playable region
 	
 	my $track_name = shift;
-	my $track = $Audio::Nama::tn{$track_name};
+	my $track = $tn{$track_name};
 	my @fades = all_fades($track_name);
+	return @fades if ! $mode->{offset_run};
 
-	
-	if($Audio::Nama::offset_run_flag){
-
-		# get end time
-		
-		my $length = Audio::Nama::wav_length($track->full_path);
-		my $play_end = Audio::Nama::play_end_time();
+	# handle offset run mode
+	my @in_bounds;
+	my $play_end = Audio::Nama::play_end_time();
+	my $play_start_time = Audio::Nama::play_start_time();
+	my $length = $track->wav_length;
+	for my $fade (@fades){
 		my $play_end_time = $play_end ?  min($play_end, $length) : $length;
-
-		# get start time
-	
-		my $play_start_time = Audio::Nama::play_start_time();
-	
-		# throw away fades that are not in play region
-	
-		@fades = grep
-			{ my $time = $Audio::Nama::Mark::by_name{$_->mark1}->{time};
-					$time >= $play_start_time
-				and $time <= $play_end_time
-			} @fades 
+		my $time = $Audio::Nama::Mark::by_name{$fade->mark1}->{time};
+		push @in_bounds, $fade if $time >= $play_start_time and $time <= $play_end_time;
 	}
-
-	# sort remaining fades by unadjusted mark1 time
-	sort{ $Audio::Nama::Mark::by_name{$a->mark1}->{time} <=>
-		  $Audio::Nama::Mark::by_name{$b->mark1}->{time}
-	} @fades;
+	@in_bounds
 }
 
 # our envelope must include a straight segment from the
@@ -159,10 +141,12 @@ sub fades {
 #   (otherwise 100%)
 
 # although we can get the precise start and endpoints,
-# I'm using 0 and $track->adjusted_playat_time + track length
+# I'm using 0 and $track->shifted_playat_time + track length
 
 sub initial_level {
 	# return 0, 1 or undef
+	# 0: track starts silent
+	# 1: track starts at full volume
 	my $track_name = shift;
 	my @fades = fades($track_name) or return undef;
 	# if we fade in we'll hold level zero from beginning
@@ -185,9 +169,9 @@ sub final_pair {   # duration: .... to length
 	my $track_name = shift;
 	my $exit_level = exit_level($track_name);
 	defined $exit_level or return ();
-	my $track = $Audio::Nama::tn{$track_name};
+	my $track = $tn{$track_name};
 	(
-		$track->adjusted_playat_time + Audio::Nama::wav_length($track->full_path),
+		$track->shifted_playat_time + $track->wav_length,
 		$exit_level
 	);
 }
@@ -212,22 +196,22 @@ sub fader_envelope_pairs {
 				$marktime1 -= $fade->duration
 			} 
 		else { $fade->dumpp; die "fade processing failed" }
-		#say "marktime1: $marktime1";
-		#say "marktime2: $marktime2";
+		logpkg(__FILE__,__LINE__,'debug',"marktime1: $marktime1, marktime2: $marktime2");
 		push @specs, 
 		[ 	$marktime1, 
 			$marktime2, 
 			$fade->type, 
-			$Audio::Nama::cops{$track->fader}->{type},
+			Audio::Nama::fxn($track->fader)->type,
 		];
 }
-	# sort fades # already done! XXX
+	# sort fades -  may not need this
 	@specs = sort{ $a->[0] <=> $b->[0] } @specs;
-	#say( Audio::Nama::yaml_out( \@specs));
+	logpkg(__FILE__,__LINE__,'debug',sub{Audio::Nama::json_out( \@specs)});
 
 	my @pairs = map{ spec_to_pairs($_) } @specs;
 
-#   XXX results in bug via AUTOLOAD for EditTrack
+#   WEIRD message - try to figure this out
+#   XXX results in bug via AUTOLOAD for Edit
 #	@pairs = (initial_pair($track->name), @pairs, final_pair($track->name)); 
 
 	# add flat segments 
@@ -236,7 +220,7 @@ sub fader_envelope_pairs {
 
 
 	# prepend number of pairs;
-	unshift @pairs, (scalar @pairs / 2);
+	unshift @pairs, (scalar @pairs / 2) if @pairs;
 	@pairs;
 }
 		
@@ -249,7 +233,7 @@ sub fader_envelope_pairs {
 
 sub spec_to_pairs {
 	my ($from, $to, $type, $op) = @{$_[0]};
-	$Audio::Nama::debug and say "from: $from, to: $to, type: $type";
+	logpkg(__FILE__,__LINE__,'debug',"from: $from, to: $to, type: $type");
 	my $cutpos;
 	my @pairs;
 
@@ -258,11 +242,11 @@ sub spec_to_pairs {
 	
 	if ($op eq 'eadb'){
 		if ( $type eq 'out' ){
-			$cutpos = $from + $fade_time1_fraction * ($to - $from);
-			push @pairs, ($from, 1, $cutpos, $fade_down_fraction, $to, 0);
+			$cutpos = $from + $config->{fade_time1_fraction} * ($to - $from);
+			push @pairs, ($from, 1, $cutpos, $config->{fade_down_fraction}, $to, 0);
 		} elsif( $type eq 'in' ){
-			$cutpos = $from + $fade_time2_fraction * ($to - $from);
-			push @pairs, ($from, 0, $cutpos, $fade_down_fraction, $to, 1);
+			$cutpos = $from + $config->{fade_time2_fraction} * ($to - $from);
+			push @pairs, ($from, 0, $cutpos, $config->{fade_down_fraction}, $to, 1);
 		}
 	}
 
@@ -298,7 +282,7 @@ sub remove_by_index {
 
 sub remove { 
 	my $fade = shift;
-	my $track = $Audio::Nama::tn{$fade->track};
+	my $track = $tn{$fade->track};
 	my $i = $fade->n;
 	
 	# remove object from index
@@ -308,30 +292,50 @@ sub remove {
 	
 	my @track_fades = all_fades($fade->track);
 	if ( ! @track_fades ){ 
-		Audio::Nama::remove_effect($track->fader);
-		$Audio::Nama::tn{$fade->track}->set(fader => undef);
+		remove_effect($track->fader);
+		$tn{$fade->track}->set(fader => undef);
 	}
 	else { refresh_fade_controller($track) }
 }
 sub add_fader {
+	# if it is missing
+
 	my $name = shift;
-	my $track = $Audio::Nama::tn{$name};
+	my $track = $tn{$name};
 
 	my $id = $track->fader;
 
-	# create a fader if necessary
+	# create a fader if necessary, place before first effect
+	# if it exists
 	
-	if (! $id){	
-		
+	if (! $id or ! Audio::Nama::fxn($id)){	
 		my $first_effect = $track->ops->[0];
-		if ( $first_effect ){
-			$id = Audio::Nama::Text::t_insert_effect($first_effect, $fader_op, [0]);
-		} else { 
-			$id = Audio::Nama::Text::t_add_effect($fader_op, [0]) 
-		}
+		$id = add_effect({
+				before 	=> $first_effect, 
+				track	=> $track,
+				type	=> $config->{fader_op}, 
+				params 	=> [0], # XX hardcoded for -ea chain operator
+		});
 		$track->set(fader => $id);
 	}
 	$id
 }
+package Audio::Nama;
+
+sub fade_uses_mark {
+	my $mark_name = shift;
+	grep{ $_->mark1 eq $mark_name or $_->mark2 eq $mark_name } values %Audio::Nama::Fade::by_index;
+}
+	
+sub apply_fades { 
+	# + data from Fade objects residing in %Audio::Nama::Fade::by_name
+	# + apply to tracks 
+	#     * that are part of current chain setup
+	#     * that have a fade operator (i.e. most user tracks)
+	map{ Audio::Nama::Fade::refresh_fade_controller($_) }
+	grep{$_->{fader} }
+	Audio::Nama::ChainSetup::engine_tracks();
+}
+	
 
 1;
\ No newline at end of file
